//==========================================================================
// File : AuxClasses.cpp
//
// Implementation of class CMyMRUFileList. Derived from CRecentFileList, 
//                         CRecentFileList is implemented in Filelist.cpp, afxadv.h
//
// Author : J. Soto
//
//==========================================================================
#include "stdafx.h"
#include "AuxClasses.h"
#include <afxpriv.h>
#include "resource.h"
#include "io.h"
//#include <dlgs.h> 


static BOOL IsWin2000 () 
{
   OSVERSIONINFOEX osvi;
   BOOL bOsVersionInfoEx;

   // Try calling GetVersionEx using the OSVERSIONINFOEX structure,
   // which is supported on Windows 2000.
   //
   // If that fails, try using the OSVERSIONINFO structure.

   ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

   if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
   {
      // If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.

      osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
      if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
         return FALSE;
   }

   switch (osvi.dwPlatformId)
   {
      case VER_PLATFORM_WIN32_NT:

         if ( osvi.dwMajorVersion >= 5 )
            return TRUE;

         break;
   }
   return FALSE; 
}


///////////////////////////////////////////////////////////////////////////
// CFileDialogEx

IMPLEMENT_DYNAMIC(CFileDialogEx, CFileDialog)

CFileDialogEx::CFileDialogEx(BOOL bOpenFileDialog,
   LPCTSTR lpszDefExt,
   LPCTSTR lpszFileName,
   DWORD dwFlags, LPCTSTR lpszFilter, CWnd* pParentWnd) :
   CFileDialog(bOpenFileDialog, lpszDefExt, lpszFileName,
      dwFlags, lpszFilter, pParentWnd)
{
}

BEGIN_MESSAGE_MAP(CFileDialogEx, CFileDialog)
   //{{AFX_MSG_MAP(CFileDialogEx)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()


int CFileDialogEx::DoModal()
{
   ASSERT_VALID(this);
   ASSERT(m_ofn.Flags & OFN_ENABLEHOOK);
   ASSERT(m_ofn.lpfnHook != NULL); // can still be a user hook

   // zero out the file buffer for consistent parsing later
   ASSERT(AfxIsValidAddress(m_ofn.lpstrFile, m_ofn.nMaxFile));
   DWORD nOffset = lstrlen(m_ofn.lpstrFile)+1;
   ASSERT(nOffset <= m_ofn.nMaxFile);
   memset(m_ofn.lpstrFile+nOffset, 0, 
         (m_ofn.nMaxFile-nOffset)*sizeof(TCHAR));

   // WINBUG: This is a special case for the file open/save dialog,
   //  which sometimes pumps while it is coming up but before it has
   //  disabled the main window.
   HWND hWndFocus = ::GetFocus();
   BOOL bEnableParent = FALSE;
   m_ofn.hwndOwner = PreModal();
   AfxUnhookWindowCreate();
   if (m_ofn.hwndOwner != NULL && ::IsWindowEnabled(m_ofn.hwndOwner))
   {
      bEnableParent = TRUE;
      ::EnableWindow(m_ofn.hwndOwner, FALSE);
   }

   _AFX_THREAD_STATE* pThreadState = AfxGetThreadState();
   ASSERT(pThreadState->m_pAlternateWndInit == NULL);

   if (m_ofn.Flags & OFN_EXPLORER)
      pThreadState->m_pAlternateWndInit = this;
   else
      AfxHookWindowCreate(this);

   memset(&m_ofnEx, 0, sizeof(m_ofnEx));
   memcpy(&m_ofnEx, &m_ofn, sizeof(m_ofn));
   if (IsWin2000())
      m_ofnEx.lStructSize = sizeof(m_ofnEx);

   int nResult;
   if (m_bOpenFileDialog)
      nResult = ::GetOpenFileName((OPENFILENAME*)&m_ofnEx);
   else
      nResult = ::GetSaveFileName((OPENFILENAME*)&m_ofnEx);

   memcpy(&m_ofn, &m_ofnEx, sizeof(m_ofn));
   m_ofn.lStructSize = sizeof(m_ofn);

   if (nResult)
      ASSERT(pThreadState->m_pAlternateWndInit == NULL);
   pThreadState->m_pAlternateWndInit = NULL;

   // WINBUG: Second part of special case for file open/save dialog.
   if (bEnableParent)
      ::EnableWindow(m_ofnEx.hwndOwner, TRUE);
   if (::IsWindow(hWndFocus))
      ::SetFocus(hWndFocus);

   PostModal();
   return nResult ? nResult : IDCANCEL;
}

BOOL CFileDialogEx::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
   memcpy(&m_ofn, &m_ofnEx, sizeof(m_ofn));
   m_ofn.lStructSize = sizeof(m_ofn);

   return CFileDialog::OnNotify( wParam, lParam, pResult);
}

/////////////////////////////////////////////////////////////////////////////
// CCustomFileDialog

IMPLEMENT_DYNAMIC(CCustomFileDialog, CFileDialogEx)

CCustomFileDialog::CCustomFileDialog(int iTemplate, BOOL bOpenFileDialog, LPCTSTR lpszDefExt, LPCTSTR lpszFileName,
		DWORD dwFlags, LPCTSTR lpszFilter, CWnd* pParentWnd) :
		CFileDialogEx(bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter, pParentWnd)
{
// This is called after CFileDialog constructor
//SAMPLE: turn on the template
	m_ofn.Flags |= OFN_ENABLETEMPLATE;
//SAMPLE: get the right module handle
	m_ofn.hInstance = AfxGetResourceHandle();
//SAMPLE: set the template to load
	m_iTemplate=iTemplate;
	m_ofn.lpTemplateName = MAKEINTRESOURCE(iTemplate);
}

BEGIN_MESSAGE_MAP(CCustomFileDialog, CFileDialogEx)
	//{{AFX_MSG_MAP(CCustomFileDialog)
	ON_BN_CLICKED(IDC_MYCFILEDLGBUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_MYCFILEDLGCHECK1, OnCheck1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CCustomFileDialog::OnButton1() 
{
//	MessageBox("You pressed button #1");
}

void CCustomFileDialog::OnCheck1() 
{
//	MessageBox("You pressed check #1");
	CWnd * pWnd=GetDlgItem(IDC_MYCFILEDLGCHECK1);
	m_BCheck1=((CButton *)pWnd)->GetCheck();
	
}


void CCustomFileDialog::OnInitDone() 
{
	CWnd * pWnd;

	CFileDialog::OnInitDone();

//	pWnd=GetParent()->GetDlgItem(0x410);
	if (m_iTemplate==IDD_MYCFILEDIALOG)
	{
		pWnd=GetDlgItem(IDC_MYCFILEDLGCHECK1);
//		pWnd->SetWindowText("Split output in multiple files (max 1 GB)");
		((CButton *)pWnd)->SetCheck(TRUE);
		m_BCheck1=((CButton *)pWnd)->GetCheck();
	}
}

/***************************************************************************
* CMyMRUFileList::CMyMRUFileList()
// Aux function of CMyMRUFileList
// FindMenuItembyID() will find a menu item ID from the specified
// popup menu and returns its position (0-based) in the specified 
// popup menu. It returns -1 if no such menu item ID is found.
*/

static int FindMenuItembyID(CMenu* Menu, int nID)
{
   ASSERT(Menu);
   ASSERT(::IsMenu(Menu->GetSafeHmenu()));

   UINT myID;
   int count = Menu->GetMenuItemCount();
   for (int i = 0; i < count; i++)
   {
	  myID=Menu->GetMenuItemID( i );

	  if (myID == (unsigned)nID)
         return i;
   }

   return -1;
}

/***************************************************************************
* CMyMRUFileList::CMyMRUFileList()
*
* Constructor. No default constructor.
* Uses the normal constructor of standard CRecentFileList, plus two additional
* parameters: MenuItemID (nMenuID) and a pointer to a Menu where the MRU is
* located (pMenu)
*
*/
 CMyMRUFileList::CMyMRUFileList(int nMenuID, CMenu* pMenu, UINT nStart,
		LPCTSTR lpszSection, LPCTSTR lpszEntryFormat, int nSize,
		int nMaxDispLen  ) : CRecentFileList( nStart,
		 lpszSection, lpszEntryFormat,  nSize,
		 nMaxDispLen )
{

	 m_nID=nMenuID;
	 m_pMenu=pMenu;
}

/***************************************************************************
* CMyMRUFileList::~CMyMRUFileList()
*
* Destructor
*
*/
CMyMRUFileList::~CMyMRUFileList()
{
}

/***************************************************************************
* CMyMRUFileList::CleanDeletedFiles()
*
* Cleans the non-accesible files
*
*/
void CMyMRUFileList::CleanDeletedFiles()
{
	int iMRU;

	for (iMRU = 0; iMRU < m_nSize; iMRU++)
	{
		if (_access(m_arrNames[iMRU], 0 ) == -1)
			Remove(iMRU);
	}
	UpdateAssociatedMenu();
}

/***************************************************************************
* CMyMRUFileList::UpdateAssociatedMenu()
*
* Updates the associated Menu Items with the File names.
* Derived from UpdateMenu() of CRecentFileList class
* but using the new member variables to locate the menu
*
*/

void CMyMRUFileList::UpdateAssociatedMenu()
{
	CMenu * pSubMenu;  // submenu where the ITEM is allocated.
	CMenu * pSubMenu2; // auxiliar submenu

	int iMRU;
	int count, i, count2, j, nID ;
	int nIndex; // pos in submenu

	count = m_pMenu->GetMenuItemCount();
 	nIndex=-1;

//  Locate the submenu where 
//	Try up two levels... (May be a recursive function would be better)
	for (i=0; i<count && nIndex<0; i++)
	{
		if (pSubMenu=m_pMenu->GetSubMenu(i))
		{
			nIndex=FindMenuItembyID(pSubMenu,m_nID);
			if ( nIndex <0)
			{
				count2 = pSubMenu->GetMenuItemCount();
				for (j=0; j<count2 && nIndex<0; j++)
				{
					if (pSubMenu2=pSubMenu->GetSubMenu(j))
						nIndex=FindMenuItembyID(pSubMenu2,m_nID);
					if ( nIndex >=0) pSubMenu=pSubMenu2;
				}
			}
		}
	}

	// Check if not found
	if ( nIndex <0) return;

	if (m_strOriginal.IsEmpty() && m_pMenu != NULL)
		m_pMenu->GetMenuString(m_nID, m_strOriginal, MF_BYCOMMAND);
	if (m_arrNames[0].IsEmpty())
	{
		// no MRU files
		if (!m_strOriginal.IsEmpty())
			m_pMenu->ModifyMenu(m_nID, MF_BYCOMMAND | MF_STRING ,m_nID, m_strOriginal);
		m_pMenu->EnableMenuItem(m_nID, MF_BYCOMMAND | MF_DISABLED |  MF_GRAYED );
		return;
	}

	for (iMRU = 0; iMRU < m_nSize; iMRU++)
		m_pMenu->DeleteMenu(m_nID + iMRU, MF_BYCOMMAND);

//  Even if the app is running in current folder, it is also shown.
/*	TCHAR szCurDir[_MAX_PATH];
	GetCurrentDirectory(_MAX_PATH, szCurDir);
	int nCurDir = lstrlen(szCurDir);
	ASSERT(nCurDir >= 0);
	szCurDir[nCurDir] = '\\';
	szCurDir[++nCurDir] = '\0';
*/
	CString strName;
	CString strTemp;

	nID=m_nID;
	for (iMRU = 0; iMRU < m_nSize; iMRU++)
	{
//		if (!GetDisplayName(strName, iMRU, szCurDir, nCurDir))
		if (!GetDisplayName(strName, iMRU, "", 0))
			break;

		// double up any '&' characters so they are not underlined
		LPCTSTR lpszSrc = strName;
		LPTSTR lpszDest = strTemp.GetBuffer(strName.GetLength()*2);
		while (*lpszSrc != 0)
		{
			if (*lpszSrc == '&')
				*lpszDest++ = '&';
			if (_istlead(*lpszSrc))
				*lpszDest++ = *lpszSrc++;
			*lpszDest++ = *lpszSrc++;
		}
		*lpszDest = 0;
		strTemp.ReleaseBuffer();

		// insert mnemonic + the file name
		TCHAR buf[10];
		wsprintf(buf, _T("&%d "), (iMRU+1+m_nStart) % 10);
		pSubMenu->InsertMenu(nIndex++,
			MF_STRING | MF_BYPOSITION, nID++,
			CString(buf) + strTemp);
	}

	// update end menu count
//	pCmdUI->m_nIndex--; // point to last menu added
//	pCmdUI->m_nIndexMax = pCmdUI->m_pMenu->GetMenuItemCount();

//	pCmdUI->m_bEnableChanged = TRUE;    // all the added items are enabled

}

/*
BOOL CRecentFileList::GetDisplayName(CString& strName, int nIndex,
	LPCTSTR lpszCurDir, int nCurDir, BOOL bAtLeastName) const
{
	ASSERT(lpszCurDir == NULL || AfxIsValidString(lpszCurDir, nCurDir));

	ASSERT(m_arrNames != NULL);
	ASSERT(nIndex < m_nSize);
	if (m_arrNames[nIndex].IsEmpty())
		return FALSE;

	LPTSTR lpch = strName.GetBuffer(_MAX_PATH);
	lstrcpy(lpch, m_arrNames[nIndex]);
	// nLenDir is the length of the directory part of the full path
	int nLenDir = lstrlen(lpch) - (AfxGetFileName(lpch, NULL, 0) - 1);
	BOOL bSameDir = FALSE;
	if (nLenDir == nCurDir)
	{
		TCHAR chSave = lpch[nLenDir];
		lpch[nCurDir] = 0;  // terminate at same location as current dir
		bSameDir = lstrcmpi(lpszCurDir, lpch) == 0;
		lpch[nLenDir] = chSave;
	}
	// copy the full path, otherwise abbreviate the name
	if (bSameDir)
	{
		// copy file name only since directories are same
		TCHAR szTemp[_MAX_PATH];
		AfxGetFileTitle(lpch+nCurDir, szTemp, _countof(szTemp));
		lstrcpyn(lpch, szTemp, _MAX_PATH);
	}
	else if (m_nMaxDisplayLength != -1)
	{
		// strip the extension if the system calls for it
		TCHAR szTemp[_MAX_PATH];
		AfxGetFileTitle(lpch+nLenDir, szTemp, _countof(szTemp));
		lstrcpyn(lpch+nLenDir, szTemp, _MAX_PATH-nLenDir);

		// abbreviate name based on what will fit in limited space
		_AfxAbbreviateName(lpch, m_nMaxDisplayLength, bAtLeastName);
	}
	strName.ReleaseBuffer();
	return TRUE;
}
*/