// VobBlanker.cpp : Defines the class behaviors for the application.
//
// Sectors Adjustment and GAPs creation Part


#include "stdafx.h"
#include "direct.h"
#include "sys/types.h"
#include "sys/stat.h"
#include "io.h"
#include "math.h"
#include "Winbase.h"
#include "ToolTipDialog.h"
#include "VobBlanker.h"
#include "VobBlankerDlg.h"
#include "LogDlg.h"
#include "InfoDlg.h"
#include "CellsDlg.h"
#include "Externs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define  M_PI 3.141592653589793238462643383279502884197169399

////////////////////////////////////////////////////////
// Calculates the sectors of the DVD, based on file sizes
// Fill the results in global members:
// m_iIFOSectors[nVTS], m_iMenuSectors[nVTS], m_iTitleSectors[nVTS],iStartSector[nVTS];
// Returns 0 if OK;  -1 if one IFO does not exist or -2 if one Title does not exist
//

int CVobBlankerApp::SectorsCalculate(CString csPath)
{
	int		FirstFreeSector,iIFOSectors,iMenuSectors,iTitleSectors;
	CString csAux,csAux2;
	int		nVTS;
	struct _stati64	statbuf;
	int		i, iRet;
	CString csError;

	iRet=0;
	csError="";
/// Compute all IFO and VOB sizes in sectors
	for (nVTS=1 ; nVTS<m_nIFOs+1 ; nVTS++)
	{
		csAux2.Format(_T("VTS_%02d_0.IFO"), nVTS);
		csAux=csPath+'\\'+csAux2;
		if (_stati64 ( csAux, &statbuf)==0)
			m_iIFOSectors[nVTS]= (int)(statbuf.st_size/2048);
		else 
		{
			m_iIFOSectors[nVTS]=-1;
			iRet=-1;
			csError+="\r\n";
			csError+=csAux2;
		}
		csAux2.Format(_T("VTS_%02d_0.VOB"), nVTS);
		csAux=csPath+'\\'+csAux2;
		if (_stati64 ( csAux, &statbuf)==0)
			m_iMenuSectors[nVTS]= (int)(statbuf.st_size/2048);
		else 
			m_iMenuSectors[nVTS]=-1;

		m_iTitleSectors[nVTS]=0;
		for (i=1; i<10; i++)
		{
			csAux2.Format(_T("VTS_%02d_%d.VOB"), nVTS,i);
			csAux=csPath+'\\'+csAux2;
			if (_stati64 ( csAux, &statbuf)==0)
				m_iTitleSectors[nVTS]+= (int)(statbuf.st_size/2048);
			else 
			{
				if (i!=1)
					i=20;
				else
				{
					m_iTitleSectors[nVTS]=-1;
					csError+="\r\n";
					csError+=csAux2;
				}
			}
		}
		if (m_iTitleSectors[nVTS]==-1 && iRet==0) iRet=-2;
	}

	csAux=csPath+"\\VIDEO_TS.IFO";
	m_iIFOSectors[0]=-1;
	if (_stati64 ( csAux, &statbuf)==0)
		m_iIFOSectors[0]= (int)(statbuf.st_size/2048);
	else 
		iRet=-1;

	csAux=csPath+"\\VIDEO_TS.VOB";
	if (_stati64 ( csAux, &statbuf)==0)
		m_iMenuSectors[0]= (int)(statbuf.st_size/2048);
	else 
		m_iMenuSectors[0]=-1;

	m_iTitleSectors[0]=-1;

	FirstFreeSector=0;
	for (nVTS=0 ; nVTS<m_nIFOs+1 ; nVTS++)
	{
/// Compute first sector of each VTS
		m_iStartSector[nVTS]=FirstFreeSector;
		if (m_iIFOSectors[nVTS] <0)  iIFOSectors=0;   else iIFOSectors=   m_iIFOSectors[nVTS];
		if (m_iMenuSectors[nVTS] <0) iMenuSectors=0;  else iMenuSectors=  m_iMenuSectors[nVTS];
		if (m_iTitleSectors[nVTS]<0) iTitleSectors=0; else iTitleSectors= m_iTitleSectors[nVTS];
		FirstFreeSector+= (2*iIFOSectors+iMenuSectors+iTitleSectors);
	}

	if (iRet<0)
		MyLogOrBox ("Error: Unable to locate at least one IFO or Title VOB file:" + csError );

	return iRet;
}

/////////////////////////////////////////////////////////
//// Deletes the menu files if they are unreferenced ////
//// Clears all attributes and tables if no menus    ////
/////////////////////////////////////////////////////////
int CVobBlankerApp::SectorsDeleteNullMenus(CString csPath)
{
	int nVTS;
	CString csAux, csMenu, csAux2;
	struct _stati64	statbuf;
	int		i, iRet;
	__int64	i64length,i64IFOlen;
	uchar * pIFO;
	FILE * in;
	FILE * out;
	int VTSM_PGCI,VTSM_C_ADT,VTSM_VOBU_ADMAP,VTSM_LU,MENU_PGC;
	int VTS_C_ADT,VTS_VOBU_ADMAP;
	int nLUs,nMPGCs, nMCells;
	int nLU,nPGCinLU,nPGC;
	int nStartCellsTable,nSectors;
	bool bMenuExists[MAX_VTS];
	int pVMG_VTS_ATRT,pVTS_ATRT,pEndAddress; 
	int k;


	iRet=0;
//  Go backwards to know which menus exist and which not	
	for (nVTS=m_nIFOs ; nVTS>=0 && iRet==0 ; nVTS--)
	{
		bMenuExists[nVTS]=false;

		if (nVTS==0)
			csMenu="VIDEO_TS.VOB";
		else
			csMenu.Format(_T("VTS_%02d_0.VOB"), nVTS);
		csMenu= csPath + "\\" + csMenu;
		if (_access(csMenu, 0) == 0) 
		{
	//Menu exists, so read IFO in memory
			bMenuExists[nVTS]=true;
		}
		if (nVTS==0)
			csAux2="VIDEO_TS.IFO";
		else
			csAux2.Format(_T("VTS_%02d_0.IFO"), nVTS);
		csAux=csPath+'\\'+csAux2;

		if (_stati64 ( csAux, &statbuf)==0)
			i64length= statbuf.st_size;
		else
		{
			MyLogOrBox("Unable to open IFO file: " + csAux);
			return -1;
		}

		i64length=i64length+MAX_NEWIFOSECTORS_INITIAL*2048; // Allow MAX_NEWSECTORS additional sectors

		pIFO= new uchar[(int)i64length];

		// Read IFO
		if ((in=fopen(csAux,"rb"))==NULL)
		{
			MyLogOrBox ("Unable to open input file: "+csAux);
			return -1;
		}
		for (i=0;!feof(in) && i< i64length ;i++)
			pIFO[i]=fgetc(in);
		i64IFOlen=i-1;
	
		if (in!=NULL) fclose(in);

		if (bMenuExists[nVTS]==false)
			ProcessIfoClearAttrib(pIFO, nVTS,  MENUS);
		else
///////////////////////////////////////////////////////////////////////////
//Load pointers and check if there are cells in PGCs
		{
			if (nVTS==0)
				VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xc8]);
			else
				VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xD0]);
			if ( VTSM_PGCI >= i64IFOlen)
			{
				MyLogOrBox ("Wrong IFO: VTSM_PGCI points out of file:\r\n"+csAux);
				delete [] pIFO;
				return -1;
			}

			VTSM_C_ADT=2048*GetNbytes(4,&pIFO[0xD8]);
			if ( VTSM_C_ADT >= i64IFOlen)
			{
				MyLogOrBox ("Wrong IFO: VTSM_C_ADT points out of file:\r\n"+csAux);
				delete [] pIFO;
				return -1;
			}

			VTSM_VOBU_ADMAP=2048*GetNbytes(4,&pIFO[0xDC]);
			if ( VTSM_VOBU_ADMAP >= i64IFOlen)
			{
				MyLogOrBox ("Wrong IFO: VTSM_VOBU_ADMAP points out of file:\r\n"+csAux);
				delete [] pIFO;
				return -1;
			}
			if (nVTS==0) VTS_C_ADT=0;
			else VTS_C_ADT=2048*GetNbytes(4,&pIFO[0xE0]);
			if ( VTS_C_ADT >= i64IFOlen)
			{
				MyLogOrBox ("Wrong IFO: VTS_C_ADT points out of file:\r\n"+csAux);
				delete [] pIFO;
				return -1;
			}
			if (nVTS==0) VTS_VOBU_ADMAP=0;
			else VTS_VOBU_ADMAP=2048*GetNbytes(4,&pIFO[0xE4]);
			if ( VTS_VOBU_ADMAP >= i64IFOlen)
			{
				MyLogOrBox ("Wrong IFO: VTS_VOBU_ADMAP points out of file:\r\n"+csAux);
				delete [] pIFO;
				return -1;
			}
			if (VTSM_C_ADT!=0)  
				nStartCellsTable=VTSM_C_ADT;
			else if (VTSM_VOBU_ADMAP!=0)
				nStartCellsTable=VTSM_VOBU_ADMAP;
			else if (nVTS!=0 && VTS_C_ADT!=0)
				nStartCellsTable=VTS_C_ADT;
			else if (nVTS!=0 && VTS_VOBU_ADMAP!=0)
				nStartCellsTable=VTS_VOBU_ADMAP;
			else
				nStartCellsTable= (int)((i64IFOlen/2048)*2048);
			
// Check nCells
			nLUs=nMPGCs=nMCells=0;

			if (VTSM_PGCI!=0)
				nLUs=GetNbytes(2,&pIFO[VTSM_PGCI]);
		
			for (nLU=0; nLU <nLUs; nLU++)
			{
				VTSM_LU= GetNbytes(4,&pIFO[VTSM_PGCI+0x04+(nLU+1)*8])+VTSM_PGCI;
				nPGCinLU=  GetNbytes(2,&pIFO[VTSM_LU]);
				nMPGCs+=   nPGCinLU;
				for (nPGC=0; nPGC < nPGCinLU; nPGC++)
				{
				    MENU_PGC= GetNbytes(4,&pIFO[VTSM_LU+0x04+(nPGC+1)*8])+VTSM_LU;
					nMCells+=pIFO[MENU_PGC+3];
				}

			} // For LUs

			if (nMPGCs == 0 || nMCells==0 )
			{
// Unreferenced Menu. Clear video attr, delete File and clear VTSM_C_ADT and VTSM_VOBU_ADMAP
//				Put2bytes(0,&pIFO[0x100]);
				ProcessIfoClearAttrib(pIFO, nVTS,  MENUS);

				if (nVTS==0)
				{
					Put4bytes(0,&pIFO[0xD8]);
					Put4bytes(0,&pIFO[0xDC]);
					i64IFOlen=nStartCellsTable;
				}
				else
				{
//Move needed sectors 
					if (VTS_C_ADT==0) nSectors=0;
					else nSectors=(VTS_C_ADT-nStartCellsTable)/2048;
					for (i= VTSM_C_ADT+nSectors*2048; i < i64IFOlen ; i++)
					{
						pIFO[i-nSectors*2048]=pIFO[i];
					}
					i64IFOlen-=(nSectors*2048);
// Update the pointers by nSectors(if not cero)
					Put4bytes(0,&pIFO[0xD8]);
					Put4bytes(0,&pIFO[0xDC]);
					for (i=0xE0 ; i<= 0xE4 ; i+=4)
						if (GetNbytes(4,&pIFO[i])!=0)
							Put4bytes(GetNbytes(4,&pIFO[i])-nSectors , &pIFO[i]);

				}
				if (DeleteFile(csMenu) != TRUE)
				{
					MyLogOrBox ("Error deleting Menu: "+csMenu);
					delete [] pIFO;
					return -1;
				}
				bMenuExists[nVTS]=false;
				if (nVTS==0)
				{
// If VIDEO_TS, clear all menu attributes when deleting MENUS
					pVMG_VTS_ATRT=GetNbytes(4,&pIFO[0xd0])*2048;
					for (nVTS=1 ; nVTS<m_nIFOs+1 && pVMG_VTS_ATRT!=0 ; nVTS++)
					{
						if (bMenuExists[nVTS]==false)
						{
							pVTS_ATRT=GetNbytes(4,&pIFO[pVMG_VTS_ATRT+8+4*(nVTS-1)])+pVMG_VTS_ATRT;
							pEndAddress= GetNbytes(4,&pIFO[pVTS_ATRT]);
							for (k=0; k<0x5c && k+8 <= pEndAddress;k++)
								pIFO[pVTS_ATRT+k+8]=0;
						}
					}
					nVTS=0;
				}
			}	// Unreferenced menu
		}  // Existence of Menu

// write IFO & BUP
		out=fopen(csAux,"wb");
		if (out!=NULL)
		{
			for (i=0;i<i64IFOlen;i++)
				fputc (pIFO[i],out);
			fclose(out);
		}
		else
		{
			MyLogOrBox ("Error writing IFO: "+csAux);
			delete [] pIFO;
			return -1;
		}

		if (nVTS==0)
			csAux2="VIDEO_TS.BUP";
		else
			csAux2.Format(_T("VTS_%02d_0.BUP"), nVTS);
		csAux=csPath+'\\'+csAux2;
		out=fopen(csAux,"wb");
		if (out!=NULL)
		{
			for (i=0;i<i64IFOlen;i++)
				fputc (pIFO[i],out);
			fclose(out);
		}
		else
		{
			MyLogOrBox ("Error writing BUP: "+csAux);
			delete [] pIFO;
			return -1;
		}
		delete [] pIFO;
	} // For VTS
	return iRet;
}

int	CVobBlankerApp::SectorsAdjustMenuAttributesVMGM (CString csPath, bool * bMenuCreated)
{
	CString csAux;
	struct _stati64	statbuf;
	int		i, iRet;
	__int64	i64length,i64IFOlen;
	uchar * pIFO;
	FILE * in;
	FILE * out;
	int nVTS;
	int pVMG_VTS_ATRT,pVTS_ATRT,pEndAddress;


	iRet=0;
// Read IFO in memory
	csAux=csPath+'\\'+"VIDEO_TS.IFO";

	if (_stati64 ( csAux, &statbuf)==0)
		i64length= statbuf.st_size;
	else
	{
		MyLogOrBox("Unable to open IFO file: " + csAux);
		return -1;
	}

	i64length=i64length+MAX_NEWIFOSECTORS_INITIAL*2048; // Allow MAX_NEWSECTORS additional sectors
	pIFO= new uchar[(int)i64length];

	// Read IFO
	if ((in=fopen(csAux,"rb"))==NULL)
	{
		MyLogOrBox ("Unable to open input file: "+csAux);
		return -1;
	}
	for (i=0;!feof(in) && i< i64length ;i++)
		pIFO[i]=fgetc(in);
	i64IFOlen=i-1;
	
	if (in!=NULL) fclose(in);
///////////////////////////////////7
// Get the pointers and do the changes

	pVMG_VTS_ATRT=GetNbytes(4,&pIFO[0xd0])*2048;
	for (nVTS=1; nVTS<m_nIFOs+1 && pVMG_VTS_ATRT!=0; nVTS++)
	{
		pVTS_ATRT=GetNbytes(4,&pIFO[pVMG_VTS_ATRT+8+4*(nVTS-1)])+pVMG_VTS_ATRT;
		pEndAddress= GetNbytes(4,&pIFO[pVTS_ATRT]);
		if (bMenuCreated[nVTS] == true)
// Copy only Video Attr of Menu domain  
			Put2bytes(m_iMVideoAttr[nVTS],&pIFO[pVTS_ATRT+8]);
	}

//////////////////////////////////////////////////////////
// write IFO & BUP
	out=fopen(csAux,"wb");
	if (out!=NULL)
	{
		for (i=0;i<i64IFOlen;i++)
			fputc (pIFO[i],out);
		fclose(out);
	}
	else
	{
		MyLogOrBox ("Error writing IFO: "+csAux);
		delete [] pIFO;
		return -1;
	}

	csAux=csPath+'\\'+"VIDEO_TS.BUP";
	out=fopen(csAux,"wb");
	if (out!=NULL)
	{
		for (i=0;i<i64IFOlen;i++)
			fputc (pIFO[i],out);
		fclose(out);
	}
	else
	{
		MyLogOrBox ("Error writing BUP: "+csAux);
		delete [] pIFO;
		return -1;
	}
	
	delete [] pIFO;
	return 0;
}


int	CVobBlankerApp::SectorsAddUnrefCellInMenu (CString csPath, int nVTS, int nPads)
{
	CString csAux, csMenu, csAux2;
	struct _stati64	statbuf;
	int		i,j, iRet;
	__int64	i64length,i64IFOlen;
	uchar * pIFO;
	FILE * in;
	FILE * out;
	int VTSM_PGCI,PREV_VTSM_PGCI;
	int VTSM_C_ADT,VTSM_VOBU_ADMAP;
	int VTS_C_ADT,VTS_VOBU_ADMAP;
	int nStartCellsTable,iMenuSectors;
	int nEndPrevTable;
	int iMVideoAttr;
	int nVID, VIDa;
	int nEntries,nLastByte,nEndADTTable,nEndADMAPTable,nStartADMAPTable;
	uint64 scr;
	int scr_ext;



	iRet=0;
// Read IFO in memory
	if (nVTS==0)
		csAux2="VIDEO_TS.IFO";
	else
		csAux2.Format(_T("VTS_%02d_0.IFO"), nVTS);

	csAux=csPath+'\\'+csAux2;

	if (_stati64 ( csAux, &statbuf)==0)
		i64length= statbuf.st_size;
	else
	{
		MyLogOrBox("Unable to open IFO file: " + csAux);
		return -1;
	}

	i64length=i64length+MAX_NEWIFOSECTORS_INITIAL*2048; // Allow MAX_NEWSECTORS additional sectors
	pIFO= new uchar[(int)i64length];

	// Read IFO
	if ((in=fopen(csAux,"rb"))==NULL)
	{
		MyLogOrBox ("Unable to open input file: "+csAux);
		return -1;
	}
	for (i=0;!feof(in) && i< i64length ;i++)
		pIFO[i]=fgetc(in);
	i64IFOlen=i-1;
	
	if (in!=NULL) fclose(in);
//////////////////////////////////////////////////////////////////////////
//Load pointers 
	if (nVTS==0)
		PREV_VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xc4]);
	else
	{
		PREV_VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xcc]);
		if (PREV_VTSM_PGCI==0)
			PREV_VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xc8]);
	}
	if ( PREV_VTSM_PGCI >= i64IFOlen || PREV_VTSM_PGCI <=0)
	{
		MyLogOrBox ("Wrong IFO: PREV_VTSM_PGCI points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}

	if (nVTS==0)
		VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xc8]);
	else
		VTSM_PGCI=2048*GetNbytes(4,&pIFO[0xD0]);
	if ( VTSM_PGCI >= i64IFOlen)
	{
		MyLogOrBox ("Wrong IFO: VTSM_PGCI points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}
	VTSM_C_ADT=2048*GetNbytes(4,&pIFO[0xD8]);
	if ( VTSM_C_ADT >= i64IFOlen)
	{
		MyLogOrBox ("Wrong IFO: VTSM_C_ADT points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}
		VTSM_VOBU_ADMAP=2048*GetNbytes(4,&pIFO[0xDC]);
	if ( VTSM_VOBU_ADMAP >= i64IFOlen)
	{
		MyLogOrBox ("Wrong IFO: VTSM_VOBU_ADMAP points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}
	if (nVTS==0) VTS_C_ADT=0;
	else VTS_C_ADT=2048*GetNbytes(4,&pIFO[0xE0]);
	if ( VTS_C_ADT >= i64IFOlen)
	{
		MyLogOrBox ("Wrong IFO: VTS_C_ADT points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}
	if (nVTS==0) VTS_VOBU_ADMAP=0;
	else VTS_VOBU_ADMAP=2048*GetNbytes(4,&pIFO[0xE4]);
	if ( VTS_VOBU_ADMAP >= i64IFOlen)
	{
		MyLogOrBox ("Wrong IFO: VTS_VOBU_ADMAP points out of file:\r\n"+csAux);
		delete [] pIFO;
		return -1;
	}
	if (VTSM_C_ADT!=0)  
		nStartCellsTable=VTSM_C_ADT;
	else if (VTSM_VOBU_ADMAP!=0)
		nStartCellsTable=VTSM_VOBU_ADMAP;
	else if (nVTS!=0 && VTS_C_ADT!=0)
		nStartCellsTable=VTS_C_ADT;
	else if (nVTS!=0 && VTS_VOBU_ADMAP!=0)
		nStartCellsTable=VTS_VOBU_ADMAP;
	else
		nStartCellsTable= (int)((i64IFOlen/2048)*2048);
////////////////////////////////////////////////////////////
// Fix needed for shrink
// Shrink does not like an unreferenced menu with VTSM_PGCI=0
	if (VTSM_PGCI==0)
	{
		nLastByte=GetNbytes(4,&pIFO[PREV_VTSM_PGCI+4]);
		nEndPrevTable=PREV_VTSM_PGCI+(nLastByte/2048+1)*2048;
/// Move needed 1 sector....
		for (i= (int)i64IFOlen-1; i>= nEndPrevTable ; i--)
		{
			pIFO[i+2048]=pIFO[i];
		}
		i64IFOlen+=2048;
		if (VTSM_C_ADT!=0)		VTSM_C_ADT+=2048;
		if (VTSM_VOBU_ADMAP!=0) VTSM_VOBU_ADMAP+=2048;
		if (VTS_C_ADT!=0)		VTS_C_ADT+=2048;
		if (VTS_VOBU_ADMAP!=0)	VTS_VOBU_ADMAP+=2048;
		nStartCellsTable+=2048;
// Clear the created hole
		for (i= 0; i < 2048 ; i++)
		{
			pIFO[nEndPrevTable +i]=0;
		}
// Update the pointers by nSectors(if not cero)
		if (nVTS==0)
		{
			Put4bytes(nEndPrevTable/2048 , &pIFO[0xC8]);
			for (i=0xCC ; i<= 0xDC ; i+=4)
				if (GetNbytes(4,&pIFO[i])!=0)
					Put4bytes(GetNbytes(4,&pIFO[i])+1 , &pIFO[i]);
		}
		else
		{
			Put4bytes(nEndPrevTable/2048 , &pIFO[0xD0]);
			for (i=0xD4 ; i<= 0xE4 ; i+=4)
				if (GetNbytes(4,&pIFO[i])!=0)
					Put4bytes(GetNbytes(4,&pIFO[i])+1 , &pIFO[i]);
		}
		if (nPads > 0) nPads--;
	}

//////////////////////////////////////////////////////////
// Fill simpleVOB with a cell with the right Video attributes
// Write the attributes in the Menu
// m_iMVideoAttr is already hacked. Always has a non-cero value

	iMVideoAttr=m_iMVideoAttr[nVTS];

	Put2bytes(iMVideoAttr,&pIFO[0x100]);

	FillSimpleVob(SimpleVob,iMVideoAttr);
////////////////////////////////////////////////////
///////////////Find VID to be used un unref cell
	nVID=1;
	if (VTSM_C_ADT==0)
		nEntries=0;
	else
	{
		nEntries=GetNbytes(4,&pIFO[VTSM_C_ADT+4]);
		nEntries=(nEntries-7)/12;
	}
	if (nEntries <0 )  nEntries=0;
	for (i=0;i < nEntries ; i++)
	{
		VIDa=GetNbytes(2,&pIFO[VTSM_C_ADT+8+12*i]);
		if (VIDa >= nVID) nVID=VIDa+1;
	}
//////////////////////////////////////////////////////////
//      Create/Append the menu file

	if (nVTS==0)
		csMenu="VIDEO_TS.VOB";
	else
		csMenu.Format(_T("VTS_%02d_0.VOB"), nVTS);
	csMenu= csPath + "\\" + csMenu;


	if (_stati64 ( csMenu, &statbuf)==0)
		iMenuSectors= (int)(statbuf.st_size/2048);
	else 
		iMenuSectors=0;

	out=fopen(csMenu,"ab");
	if (out!=NULL)
	{
		for (i=0;i<5; i++)
		{
			for (j=0; j< 2048; j++)
				m_buffer[j]=SimpleVob[i*2048+j];
			if (IsNav(m_buffer))
			{
				ModifyCID (m_buffer,nVID, 1);
				if (nPads > 0)
					Put4bytes(4+nPads,&m_buffer[0x40f]);
				ModifyLBA (m_buffer, iMenuSectors);
			}
			if ( writebuffer(m_buffer,out) != 2048)
			{
				MyLogOrBox ("Error writting VOB file. Check disk space\r\n"+csMenu);
				delete [] pIFO;
				return -1;
			}
		}
	}
// Output pad packs	
	FillPaddingPack(m_buffer);

	scr=readscr(&SimpleVob[4*2048]);
	scr_ext=readscr_ext(&SimpleVob[4*2048]);
	for (i=0;i<nPads ; i++)
	{
		scr+=147;
		writescr(scr,scr_ext,m_buffer);
		if ( writebuffer(m_buffer,out) != 2048)
		{
			MyLogOrBox ("Error writting VOB file. Check disk space\r\n"+csMenu);
			delete [] pIFO;
			return -1;
		}
	}
	fclose(out);
//////////////////////////////////////////////////////////
//     Modify IFO: TABLE VTSM_C_ADT
	if (VTSM_C_ADT==0)
		nEntries=nLastByte=0;
	else
	{
		nLastByte=GetNbytes(4,&pIFO[VTSM_C_ADT+4]);
		nEntries=(nLastByte-7)/12;
	}
	if (nEntries <0 )  nEntries=0;
	if ( ( (nLastByte/2048) < ((nLastByte+12)/2048) ) || nEntries==0)
	{
/// Move needed 1 sector....
		if (nEntries==0) nEndADTTable=nStartCellsTable;
		else nEndADTTable=nStartCellsTable+(nLastByte/2048+1)*2048;
		for (i= (int)i64IFOlen-1; i>= nEndADTTable ; i--)
		{
			pIFO[i+2048]=pIFO[i];
		}
		i64IFOlen+=2048;
// Clear the created hole
		for (i= 0; i < 2048 ; i++)
		{
			pIFO[nEndADTTable +i]=0;
		}
// Update the pointers by nSectors(if not cero)

		if (VTSM_C_ADT==0)	Put4bytes(nStartCellsTable/2048 , &pIFO[0xD8]);
		for (i=0xDC ; i<= 0xE4 ; i+=4)
			if (GetNbytes(4,&pIFO[i])!=0)
				Put4bytes(GetNbytes(4,&pIFO[i])+1 , &pIFO[i]);
		VTSM_C_ADT=nStartCellsTable;
	}
// Fill the new cell parameters
// Lastbyte
	if (nLastByte==0) nLastByte=19;
	else nLastByte+=12;
	Put4bytes(nLastByte,&pIFO[VTSM_C_ADT+4]);
// # VOBs
	Put2bytes(GetNbytes(2,&pIFO[VTSM_C_ADT])+1,&pIFO[VTSM_C_ADT]);
// VID/CID	
	Put2bytes(nVID,&pIFO[VTSM_C_ADT+8+12*nEntries]);
	pIFO[VTSM_C_ADT+8+12*nEntries+2]=1;
	pIFO[VTSM_C_ADT+8+12*nEntries+3]=0;
// start and end sectors
	Put4bytes(iMenuSectors,        &pIFO[VTSM_C_ADT+8+12*nEntries+4]);
	Put4bytes(iMenuSectors+nPads+4,&pIFO[VTSM_C_ADT+8+12*nEntries+8]);

// New end of ADT table
	nLastByte=GetNbytes(4,&pIFO[VTSM_C_ADT+4]);
	nEndADTTable=nStartCellsTable+(nLastByte/2048+1)*2048;
//////////////////////////////////////////////////////////
//     Modify IFO: TABLE VTSM_VOBU_ADMAP

	if (VTSM_VOBU_ADMAP==0)
		nEntries=nLastByte=0;
	else
	{
		nLastByte=GetNbytes(4,&pIFO[VTSM_VOBU_ADMAP]);
		nEntries=(nLastByte-3)/4;
	}
	if (nEntries <0 )  nEntries=0;
	if ( ( (nLastByte/2048) < ((nLastByte+4)/2048) ) || nEntries==0)
	{
/// Move needed 1 sector....
		nStartADMAPTable=nEndADTTable;
		if (nEntries==0) nEndADMAPTable=nEndADTTable;
		else nEndADMAPTable=nStartADMAPTable+(nLastByte/2048+1)*2048;
		for (i= (int)i64IFOlen-1; i>= nEndADMAPTable ; i--)
		{
			pIFO[i+2048]=pIFO[i];
		}
		i64IFOlen+=2048;
// Clear the created hole
		for (i= 0; i < 2048 ; i++)
		{
			pIFO[nEndADMAPTable +i]=0;
		}
// Update the pointers by nSectors(if not cero)

		if (VTSM_VOBU_ADMAP==0)	Put4bytes(nStartADMAPTable/2048 , &pIFO[0xDC]);
		for (i=0xE0 ; i<= 0xE4 ; i+=4)
			if (GetNbytes(4,&pIFO[i])!=0)
				Put4bytes(GetNbytes(4,&pIFO[i])+1 , &pIFO[i]);
		VTSM_VOBU_ADMAP=nStartADMAPTable;
	}
// Fill the new cell parameters
// Lastbyte
	if (nLastByte==0) nLastByte=7;
	else nLastByte+=4;
	Put4bytes(nLastByte,&pIFO[VTSM_VOBU_ADMAP]);
// # Pointer to VOBU
	Put4bytes(iMenuSectors,&pIFO[VTSM_VOBU_ADMAP+4+4*nEntries]);


//////////////////////////////////////////////////////////
// write IFO & BUP
	out=fopen(csAux,"wb");
	if (out!=NULL)
	{
		for (i=0;i<i64IFOlen;i++)
			fputc (pIFO[i],out);
		fclose(out);
	}
	else
	{
		MyLogOrBox ("Error writing IFO: "+csAux);
		delete [] pIFO;
		return -1;
	}

	if (nVTS==0)
		csAux2="VIDEO_TS.BUP";
	else
		csAux2.Format(_T("VTS_%02d_0.BUP"), nVTS);
	csAux=csPath+'\\'+csAux2;
	out=fopen(csAux,"wb");
	if (out!=NULL)
	{
		for (i=0;i<i64IFOlen;i++)
			fputc (pIFO[i],out);
		fclose(out);
	}
	else
	{
		MyLogOrBox ("Error writing BUP: "+csAux);
		delete [] pIFO;
		return -1;
	}
	
	delete [] pIFO;
	return 0;
}



int CVobBlankerApp::SectorsCreateGaps(CString csPath)
{

	CString csAux,csAux2;
	int		nVTS;
	double  dLinearSize, dA, dAngle;
	int		nTrack,nSectorsPerTrac,nBytesPerTrac,nSectorsGAP, nSectorsRemain;
	int		iIFOSectors,iMenuSectors,iTitleSectors;
	int		iRet;
	bool	bMenuCreated[MAX_VTS];


	iRet=SectorsCalculate(csPath);
	if (iRet !=0) return iRet;

	for (nVTS=0 ; nVTS<m_nIFOs+1 &&iRet==0 ; nVTS++)
	{
//		bMenuCreated[nVTS]=false;
		if (m_iIFOSectors[nVTS] <0)  iIFOSectors=0;   else iIFOSectors=   m_iIFOSectors[nVTS];
		if (m_iMenuSectors[nVTS] <0) iMenuSectors=0;  else iMenuSectors=  m_iMenuSectors[nVTS];
		if (m_iTitleSectors[nVTS]<0) iTitleSectors=0; else iTitleSectors= m_iTitleSectors[nVTS];
//// Calculate the number of track for iStartSector
		dLinearSize=0.0001333*1024*16*1209/512*m_iStartSector[nVTS];
		dA=0.00074/(4*M_PI);
//		dB=24.0;
//		dC=-1*dLinearSize;
		dAngle=(-24.0 + sqrt (24.0*24.0 + 4*dA*dLinearSize))/2/dA;
		nTrack= (int) (dAngle/(2*M_PI));
		nBytesPerTrac=(int)((58.48+62.78*nTrack/45946.0)*1024 );
		nSectorsPerTrac=(int)((58.48+62.78*nTrack/45946.0 + 1.0)/2.0);
		nSectorsGAP=nSectorsPerTrac/2-iIFOSectors;
		if ( iMenuSectors==0) nSectorsGAP-=2; //IFO is going to grow 2 sectors
		if ((iMenuSectors+iTitleSectors)< (nSectorsPerTrac*MINTRACKSGAP))
		{
// No enough space between BUP and IFO, trying to put the BUP in the opposite 1/2 turn
// Guarantee 32 K (16 sectors +4 tolerance)
			while (nSectorsGAP < 20 ) 
			{
				nSectorsGAP+=nSectorsPerTrac;
			}
			nSectorsRemain=(((iMenuSectors+iTitleSectors)*2048)%nBytesPerTrac)/2048;
			nSectorsRemain=(nSectorsRemain%nSectorsPerTrac);
			nSectorsGAP-=nSectorsRemain;
			while (nSectorsGAP < -3)
				nSectorsGAP+=nSectorsPerTrac;
			if (nSectorsGAP >= 5 )
			{
// Call the procedure to add an unreferenced menu (or an unreferenced cell in the menu)
				iRet=SectorsAddUnrefCellInMenu (csPath,nVTS,nSectorsGAP-5);
				if ( iMenuSectors==0 ) bMenuCreated[nVTS]=true;
			}
		}
	}	
/////////////
	if (iRet==0)
		iRet=SectorsAdjustMenuAttributesVMGM (csPath, bMenuCreated);
	return iRet;
}


int CVobBlankerApp::SectorsAdjust(CString csPath)
{
	int		iIFOSectors,iMenuSectors,iTitleSectors;
	CString csAux,csAux2;
	int		iTT_SRPT;
	int		nVTS,nT;
	int		i,iRet,nStart;
	FILE * in;

	iRet=SectorsCalculate(csPath);
	if (iRet !=0) return iRet;

	for (nVTS=1 ; nVTS<m_nIFOs+1 ; nVTS++)
	{
		if (m_iIFOSectors[nVTS] <0)  iIFOSectors=0;   else iIFOSectors=   m_iIFOSectors[nVTS];
		if (m_iMenuSectors[nVTS] <0) iMenuSectors=0;  else iMenuSectors=  m_iMenuSectors[nVTS];
		if (m_iTitleSectors[nVTS]<0) iTitleSectors=0; else iTitleSectors= m_iTitleSectors[nVTS];
//////////////////////////////////////////////////////
// SECTORS IN VTS_XX_0.IFO

		csAux.Format(_T("VTS_%02d_0.IFO"), nVTS);
		csAux=csPath+'\\'+csAux;

		csAux2.Format(_T("VTS_%02d_0.BUP"), nVTS);
		csAux2=csPath+'\\'+csAux2;

		if ((in=fopen(csAux,"r+b"))==NULL)
		{
			MyLogOrBox ("Unable to open IFO file: " + csAux);
			return -1;
		}

		fseek(in,0x1C,SEEK_SET);
		fput4c(iIFOSectors-1,in);

		fseek(in,0xC0,SEEK_SET);
		if (m_iMenuSectors[nVTS]>=0)
			fput4c(iIFOSectors,in);
		else
		{
	// if m_iMenuSectors[nVTS] < 0, means menu does not exist
			fput4c(0,in);
		}

		fseek(in,0xC4,SEEK_SET);
		fput4c(iIFOSectors+iMenuSectors,in);

		fseek(in,0x0C,SEEK_SET);
		fput4c(2*iIFOSectors+iMenuSectors+iTitleSectors-1,in);

// Remove POUPs in Skipped IFOs: Original pointers can be used
// VIDEO_TS.IFO is always processed, so no need to remove PUOPs again
		if (m_bCheckIfoPuops && m_iActionVTS[nVTS]==SKIP )
		{
			for (nT=0;nT<m_nPGCs[nVTS];nT++)
			{
				fseek(in, m_VTS_PGC[nVTS][nT]+8 ,SEEK_SET);
				fput4c( 0 ,in);
			}

			for (nT=0;nT<m_nMPGCs[nVTS];nT++)
			{
				fseek(in, m_MENU_PGC[nVTS][nT]+8 ,SEEK_SET);
				fput4c( 0 ,in);
			}
		}
		fclose(in);
//		MyLogOrBox (csAux + " modified. Updating BUP");
		CopyFile(csAux, csAux2,FALSE);
	}

//////////////////////////////////////////////////////
// SECTORS IN VIDEO_TS.IFO

// OWN SECTORS
	if (m_iIFOSectors[0] <0)  iIFOSectors=0;   else iIFOSectors=   m_iIFOSectors[0];
	if (m_iMenuSectors[0] <0) iMenuSectors=0;  else iMenuSectors=  m_iMenuSectors[0];
	if (m_iTitleSectors[0]<0) iTitleSectors=0; else iTitleSectors= m_iTitleSectors[0];

	csAux=csPath+"\\VIDEO_TS.IFO";
	csAux2=csPath+"\\VIDEO_TS.BUP";
	
	if ((in=fopen(csAux,"r+b"))==NULL)
	{
		MyLogOrBox("Unable to open IFO file: " + csAux);
		return -1;
	}

	fseek(in,0x1C,SEEK_SET);
	fput4c(iIFOSectors-1,in);

	fseek(in,0xC0,SEEK_SET);
	if (m_iMenuSectors[0]>=0)
		fput4c(iIFOSectors,in);
	else
	{
	// Menu does not exist
		fput4c(0,in);
	}

	fseek(in,0x0C,SEEK_SET);
	fput4c(2*iIFOSectors+iMenuSectors-1,in);
	

// TT_SRPT: Table of titles
///////////////////////////////////////////////////////
// Titles loop

	fseek(in,0x0C4,SEEK_SET);
	iTT_SRPT=2048*get4c(in);

	for (i=0;i< m_nTotalTitles ; i++)
	{
		if (fseek(in,iTT_SRPT+8+12*i+6,SEEK_SET)==0)
		{
			nStart=getc(in);
			if (nStart !=EOF && nStart < m_nIFOs+1 && nStart >0)
			{
				fseek(in,iTT_SRPT+8+12*i+8,SEEK_SET);
				fput4c(m_iStartSector[nStart],in);
			}
			else
			{
				MyLogOrBox ("Unable to adjust all title sectors");
				fclose(in);
				return -1;
			}
		}
		else
		{
			MyLogOrBox ("Unable to get title number in VIDEO_TS.IFO");
			fclose(in);
			return -1;
		}
	}

	fclose(in);
	CopyFile(csAux, csAux2,FALSE);
	return 0;
}