#include "stdafx.h"

typedef unsigned char uchar;
typedef unsigned __int64 uint64;
#include "FixedVobs.h"
#include "Languages.h"

#define PAL 1
#define NTSC 0

bool MyDeleteFile(const char *szFilename)
// Taken from Autoit: Accepts wildcards //
{
	WIN32_FIND_DATA	findData;
	HANDLE			hSearch;
	bool			bFound = false;				// Not found initially
	bool			bLoop;
	char			szDrive[_MAX_PATH+1];
	char			szDir[_MAX_PATH+1];
	char			szFile[_MAX_PATH+1];
	char			szExt[_MAX_PATH+1];
	char			szTempPath[_MAX_PATH+1];

	strcpy(szTempPath,szFilename);
	// Split the target into bits (used for reconstruction later)
	_splitpath( szTempPath, szDrive, szDir, szFile, szExt );

	// Delete all files matching the criteria
	hSearch = FindFirstFile(szTempPath, &findData);
	bLoop = true;
	while (hSearch != INVALID_HANDLE_VALUE && bLoop == true)
	{
		// Make sure the returned handle is a file and not a directory before we
		// try and do delete type things on it!
		if ( (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY)
		{
			bFound = true;							// Found at least one match
			
			// Returned data is just the filename, we need to add the path stuff back on
			// The find strcut only returns the file NAME, we need to reconstruct the path!
			strcpy(szTempPath, szDrive);	
			strcat(szTempPath, szDir);
			strcat(szTempPath, findData.cFileName);
			
			if ( DeleteFile(szTempPath) != TRUE )
			{
				FindClose(hSearch);
				return false;						// Error deleting one of the files
			}

		} // End if

		if (FindNextFile(hSearch, &findData) == FALSE)
			bLoop = false;
	}

	FindClose(hSearch);

	return bFound;
}

int get4c(FILE * in)
{
	int iRet;
	int i1,i2,i3,i4;

	i1=fgetc(in);
	i2=fgetc(in);
	i3=fgetc(in);
	i4=fgetc(in);

	if (i4!=EOF) 
		iRet=i1*256*256*256+i2*256*256+i3*256+i4;
	else iRet=0;

	return	iRet;
}

void fput4c( int i4char, FILE *in)
{
	fputc ( (i4char/(256*256*256)), in);
	fputc ( (i4char/(256*256))%256, in);
	fputc ( (i4char/256)%256, in);
	fputc ( (i4char)%256, in);
}


CString lang_long(char * szLang)
{
	int j;
	CString csAux;

	for (j=0,csAux="Unknown";LangTable[j].descr[0]!=0 && csAux=="Unknown";j++)
	{
		if (szLang[0]==LangTable[j].descr[0] &&
			szLang[1]==LangTable[j].descr[1])
			 csAux=  LangTable[j].lang_long;
	}

	return csAux;
}

void MyErrorBox(LPCTSTR text)
{
	AfxMessageBox(text, MB_OK|MB_ICONSTOP,0);
}

void MyInfoBox(LPCTSTR text)
{
	AfxMessageBox(text,MB_OK|MB_ICONINFORMATION,0);
}

UINT GetNbytes(int nNumber,uchar* address)
{
	UINT ret;
	int i;

	for (i=0,ret=0;i<nNumber;i++)
		ret=ret*256+address[i];
	return ret;
}

void Put4bytes(__int64 i64Number,uchar* address)
{

	address[0]= (uchar) (i64Number/(256*256*256));
	address[1]= (uchar) ((i64Number/(256*256))%256);
	address[2]= (uchar) ((i64Number/256)%256);
	address[3]= (uchar) (i64Number%256);

}

void Put2bytes(__int64 i64Number,uchar* address)
{

	address[0]= (uchar) ((i64Number/256)%256);
	address[1]= (uchar) (i64Number%256);

}

uint64 readpts(uchar *buf)
{
    uint64 a1,a2,a3;
	uint64 pts;

    a1=(buf[0]&0xe)>>1;
    a2=((buf[1]<<8)|buf[2])>>1;
    a3=((buf[3]<<8)|buf[4])>>1;
    pts= ( (a1<<30) | (a2<<15) | a3);
    return pts;
}


void writepts(uchar *buf,uint64 pts)
{
    buf[0]=(uchar)(((pts>>29)&0xe))|(buf[0]&0xf1); // this preserves the PTS / DTS / PTSwDTS top bits
    Put2bytes((pts>>14)|1, buf+1);
    Put2bytes((pts<<1)|1, buf+3);
}


uint64 readscr(uchar *buffer)
{
	uint64 scr0,scr1,scr2,scr;

	scr0=(uint64) GetNbytes(3,&buffer[6]);
	scr0=(scr0 & 0x0003FFF8) >>3;
	scr1=(uint64) GetNbytes(3,&buffer[4]);
	scr1=(scr1 & 0x0003FFF8) >>3;
	scr2=(uint64) buffer[4];
	scr2=(scr2  & 0x38 )>>3;
		
	scr= scr0 + (scr1<<15) + ( scr2<<30 );

	return scr;
}

int readscr_ext(uchar *buffer)
{
	int scr_ext;

	scr_ext=(int) GetNbytes(2,&buffer[8]);
	scr_ext= (scr_ext >> 1) & 0x1FF;

	return scr_ext;
}

void writescr(uint64 scr, int scr_ext, uchar *buffer)
{
	uint64 scr0,scr1,scr2,scre, scr_all;

	scr0= (scr & (uint64)0x0000007FFF);
	scr1= (scr & (uint64)0x003FFF8000);
	scr2= (scr & (uint64)0x01C0000000);
	scre= (scr_ext & 0x1FF);

	scr_all= (scre<<1) + (scr0<<11) + (scr1<<12) + ( scr2<<13 )+ (uint64)0x440004000401;

	Put4bytes( (scr_all >>16 ), &buffer[4]);
	Put2bytes( (scr_all & 0x0FFFF ), &buffer[8]);
}


CString Size_MBytes(__int64 i64size)
{
	CString csAux;

	if (i64size < 9999)
		csAux.Format(_T("%I64d By"),i64size);
	else if (i64size < 9999*1024)
		csAux.Format(_T("%I64d KB"),i64size/1024);
	else if (i64size < (__int64)9999*1024*1024)
		csAux.Format(_T("%I64d MB"),i64size/1024/1024);
	else
		csAux.Format(_T("%I64d MB"),i64size/1024/1024/1024);
	
	return csAux;


}

CString FormatDuration(DWORD duration)
{
	CString csAux;

	csAux.Format(_T("%02X:%02X:%02X.%02X"),
		duration/(256*256*256),(duration/(256*256))%256,(duration/256)%256,(duration%256) & 0x3f);

	return csAux;
}

int BCD2Dec ( int BCD)
{
	int ret;
	ret = (BCD/0x10)*10 + (BCD%0x10);
	return ret;
}

int Dec2BCD ( int Dec)
{
	int ret;
	ret = (Dec/10)*0x10 + (Dec%10);
	return ret;
}

int DurationInFrames(DWORD dwDuration)
{

	int ifps,ret;
	__int64 i64Dur;

	if ( ((dwDuration%256) & 0x0c0) == 0x0c0 )
		ifps=30;
	else  
		ifps=25;
	
	i64Dur=  (BCD2Dec( (dwDuration%256) & 0x3f));
	i64Dur+= (BCD2Dec( (dwDuration/256)%256 )*ifps);
	i64Dur+= (BCD2Dec( (dwDuration/(256*256))%256 )*ifps*60);
	i64Dur+= (BCD2Dec(  dwDuration/(256*256*256)  )*ifps*60*60);

	ret=(int)(i64Dur);

	return ret;
}

int DurationInSecs(DWORD dwDuration)
{

	int ifps,ret;
	__int64 i64Dur;

	if ( ((dwDuration%256) & 0x0c0) == 0x0c0 )
		ifps=30;
	else  
		ifps=25;
	
	i64Dur=  (BCD2Dec( (dwDuration%256) & 0x3f));
	i64Dur+= (BCD2Dec( (dwDuration/256)%256 )*ifps);
	i64Dur+= (BCD2Dec( (dwDuration/(256*256))%256 )*ifps*60);
	i64Dur+= (BCD2Dec(  dwDuration/(256*256*256)  )*ifps*60*60);

	ret=(int)(i64Dur/ifps);

	return ret;
}

DWORD AddDuration(DWORD dwDuration1, DWORD dwDuration2)
{
	DWORD ret;
	int ifps,hh,mm,ss,ff;
	__int64 i64Dur1, i64Dur2, i64DurT;

	if ( ((dwDuration1%256) & 0x0c0) == 0x0c0 )
		ifps=30;
	else  
		ifps=25;
	
	i64Dur1=  BCD2Dec( (dwDuration1%256) & 0x3f);
	i64Dur1+= BCD2Dec( (dwDuration1/256)%256 )*ifps;
	i64Dur1+= BCD2Dec( (dwDuration1/(256*256))%256 )*ifps*60;
	i64Dur1+= BCD2Dec( dwDuration1/(256*256*256) )*ifps*60*60;

	i64Dur2=  BCD2Dec( (dwDuration2%256) & 0x3f);
	i64Dur2+= BCD2Dec( (dwDuration2/256)%256 )*ifps;
	i64Dur2+= BCD2Dec( (dwDuration2/(256*256))%256 )*ifps*60;
	i64Dur2+= BCD2Dec( dwDuration2/(256*256*256) )*ifps*60*60;

	i64DurT=i64Dur2+i64Dur1;

	ff=Dec2BCD( (int)(i64DurT%ifps)     );
	ss=Dec2BCD( (int)((i64DurT/ifps)%60)   );
	mm=Dec2BCD( (int)((i64DurT/ifps/60)%60) );
	hh=Dec2BCD( (int)(i64DurT/ifps/60/60)  );

	ret=ff + ss*256 + mm*256*256 + hh*256*256*256;

	if (ifps==30)
		ret+= 0x0c0;
	else
		ret+= 0x040;

	return ret;
}

DWORD SubDuration(DWORD dwDuration1, DWORD dwDuration2)
{
	DWORD ret;
	int ifps,hh,mm,ss,ff;
	__int64 i64Dur1, i64Dur2, i64DurT;

	if ( ((dwDuration1%256) & 0x0c0) == 0x0c0 )
		ifps=30;
	else  
		ifps=25;
	
	i64Dur1=  BCD2Dec( (dwDuration1%256) & 0x3f);
	i64Dur1+= BCD2Dec( (dwDuration1/256)%256 )*ifps;
	i64Dur1+= BCD2Dec( (dwDuration1/(256*256))%256 )*ifps*60;
	i64Dur1+= BCD2Dec( dwDuration1/(256*256*256) )*ifps*60*60;

	i64Dur2=  BCD2Dec( (dwDuration2%256) & 0x3f);
	i64Dur2+= BCD2Dec( (dwDuration2/256)%256 )*ifps;
	i64Dur2+= BCD2Dec( (dwDuration2/(256*256))%256 )*ifps*60;
	i64Dur2+= BCD2Dec( dwDuration2/(256*256*256) )*ifps*60*60;

	i64DurT=i64Dur1-i64Dur2;

	ff=Dec2BCD( (int)(i64DurT%ifps)     );
	ss=Dec2BCD( (int)((i64DurT/ifps)%60)   );
	mm=Dec2BCD( (int)((i64DurT/ifps/60)%60) );
	hh=Dec2BCD( (int)(i64DurT/ifps/60/60)  );

	ret=ff + ss*256 + mm*256*256 + hh*256*256*256;

	if (ifps==30)
		ret+= 0x0c0;
	else
		ret+= 0x040;

	return ret;
}

// Sorry, a dirty trick to pass the folder 
CString csGlobalStartFolder;

int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData)
{
	TCHAR szDir[MAX_PATH];

         switch(uMsg)
		 {
            case BFFM_INITIALIZED: {
               //if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR), szDir))
				if (csGlobalStartFolder.IsEmpty())
                  strcpy(szDir,"C:\\");
				else
                  strcpy(szDir,csGlobalStartFolder);
			   {
                  // WParam is TRUE since you are passing a path.
                  // It would be FALSE if you were passing a pidl.
                  SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
               }
               break;
            }
            case BFFM_SELCHANGED:
			{
               // Set the status window to the currently selected path.
               if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
			   {
                  SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
               }
             break;
            }
            default:
             break;
         }
         return 0;
}


CString SelectFolder (CString csTitulo, CString csStartFolder)
{
	BROWSEINFO bi;
    LPITEMIDLIST pidlBrowse;    
	char	buffer[_MAX_PATH];
	char	path[_MAX_PATH];

	csGlobalStartFolder=csStartFolder;
	bi.hwndOwner = AfxGetMainWnd()->m_hWnd;     
	bi.pidlRoot = NULL; 
	bi.pszDisplayName = buffer;     
	bi.lpszTitle = (const char*)csTitulo; 
	bi.ulFlags = BIF_EDITBOX|  0x40 | BIF_RETURNONLYFSDIRS ;
	//BIF_USENEWUI;     
    bi.lpfn = BrowseCallbackProc;
//	bi.lpfn = NULL;     
	bi.lParam = 0;  
	bi.iImage = 0;
	pidlBrowse = SHBrowseForFolder(&bi);     
	if (pidlBrowse != NULL) 
	{ 
		SHGetPathFromIDList(pidlBrowse,path);
		return CString(path);
	}
	return "";
}


/*

CString SelectFolder (CString csTitulo)
{

//    LPITEMIDLIST pidlRoot = NULL;
//    LPMALLOC pMalloc = NULL;
	BROWSEINFO bi = {0};
    LPITEMIDLIST pidlBrowse=NULL;    
	char	buffer[_MAX_PATH];
	char	path[_MAX_PATH];


    pidlRoot = NULL;
//  SHGetSpecialFolderLocation(NULL,CSIDL_DRIVES, &pidlRoot);
    
	bi.hwndOwner = AfxGetMainWnd()->m_hWnd;     
	bi.pidlRoot = pidlRoot; 
	bi.pszDisplayName = buffer;     
	bi.lpszTitle = (const char*)csTitulo; 
	bi.ulFlags = BIF_EDITBOX |  0x40 | BIF_RETURNONLYFSDIRS ;
//	bi.ulFlags = 0x200;
	bi.lpfn = NULL;     
	bi.lParam = 0;  
	bi.iImage = 0;

	pidlBrowse = SHBrowseForFolder(&bi);     
	if (pidlBrowse != NULL) 
	{ 
		SHGetPathFromIDList(pidlBrowse,path);
		return CString(path);
	}
	return "";
}
*/

int readbuffer(uchar *caracter, FILE *in)
{
	int j;

	if (in == NULL) return -1;
	j =fread (caracter,sizeof (uchar), 2048, in);

	return j;
}


int writebuffer(uchar *caracter, FILE *out)
{
	int nwritten;

	nwritten=fwrite (caracter,sizeof(uchar),2048,out);

	return nwritten;
}


void writetempbuffer(uchar *buffer, uchar *MegaBuffer, int iMega)
{
	memcpy (&MegaBuffer[iMega*2048],buffer, 2048);
}

int FlushMegaBuffer(uchar *MegaBuffer, int m_nPackstobeWritten, FILE *out)
{
	int nwritten;

	nwritten=fwrite (MegaBuffer,sizeof(uchar),m_nPackstobeWritten*2048,out);

	return nwritten;
}


__int64 VB_GetFreeSpaceInMB(CString csFolder)
{
	__int64 i64Free, i64Total;
	int iRet;

	iRet=GetDiskFreeSpaceEx( csFolder,
			(PULARGE_INTEGER)&i64Free,
            (PULARGE_INTEGER)&i64Total,
            (PULARGE_INTEGER)NULL);

	if (iRet!=0)
		return i64Free/1024/1024;
	else
		return -1;
}


bool IsNav(uchar* buffer)
{

	int startcode, stuffing;

//  VobBlanker does not support stuffing in Nav. If it is there,
//	all the code is wrong, so better check it
	stuffing=buffer[13]&7;

	if (stuffing!=0) return false;

	startcode=GetNbytes(4, &buffer[14]);

	if (startcode==443) return true;
	else return false;

}

bool IsVideo (uchar* buffer)
{

	int startcode, stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);

	if (startcode==480) return true;
	else return false;

}
int GetStartGOP(uchar* buffer)
{
	int startcode, startmpeg;
	int j;

	if (!IsVideo(buffer)) return -1;

	startmpeg=0x17+buffer[0x16];
	startcode=GetNbytes(4, &buffer[startmpeg]);
	if (startcode!=0x1b3) return -1;

	for (j=startmpeg; j<2048-4; j++)
	{
		if (buffer[j]   == 0 && buffer[j+1] == 0 &&
		    buffer[j+2] == 1 && buffer[j+3] == 0xb8 )
		{
			return j;
		}
	}		

	return -1;
}
bool IsGOP (uchar* buffer)
{

	int startcode, startmpeg;

	startmpeg=0x17+buffer[0x16];
	startcode=GetNbytes(4, &buffer[startmpeg]);

	if (startcode==0x1b3)
	{
		if (GetStartGOP(buffer)>0)
			return true;
	}
	
	return false;
}

bool IsClosedGOP (uchar* buffer)
{

	int startcode, startmpeg;
	int startGOP, GOPflags;

	startmpeg=0x17+buffer[0x16];
	startcode=GetNbytes(4, &buffer[startmpeg]);

	if (startcode==0x1b3)
	{
		startGOP=GetStartGOP(buffer);
		if (startGOP>0)
		{
			GOPflags=GetNbytes(4, &buffer[startGOP+4]);
			if (GOPflags & 0x40) 
				return true;
		}
	}
	return false;
}

bool SetBreakGOP (uchar* buffer)
{

	int startcode, startmpeg;
	int startGOP; 
	UINT GOPflags;

	startmpeg=0x17+buffer[0x16];
	startcode=GetNbytes(4, &buffer[startmpeg]);

	if (startcode==0x1b3)
	{
		startGOP=GetStartGOP(buffer);
		if (startGOP>0)
		{
			GOPflags=GetNbytes(4, &buffer[startGOP+4]);
			GOPflags=(GOPflags & (UINT)(0xFFFFFFBF) ) | 0x20; 
			Put4bytes(GOPflags,&buffer[startGOP+4]);
			return true;
		}
	}
	return false;
}

bool SetClosedGOP (uchar* buffer)
{

	int startcode, startmpeg;
	int startGOP; 
	UINT GOPflags;

	startmpeg=0x17+buffer[0x16];
	startcode=GetNbytes(4, &buffer[startmpeg]);

	if (startcode==0x1b3)
	{
		startGOP=GetStartGOP(buffer);
		if (startGOP>0)
		{
			GOPflags=GetNbytes(4, &buffer[startGOP+4]);
			GOPflags=(GOPflags & (UINT)(0xFFFFFFCF) ) | 0x40 ; 
			Put4bytes(GOPflags,&buffer[startGOP+4]);
			return true;
		}
	}
	return false;
}

bool IsAudio (uchar* buffer)
{
	int startcode,st_i,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);
	st_i=0x17+buffer[0x16+stuffing];
/*
0x80-0x87: ac3
0x88-0x8f: dts
0x90-0x9f: unknown
0xa0-0xa7: lpcm
*/
	if ((startcode==445 && buffer[st_i] >0x7f && buffer[st_i] < 0x90) || 
	    (startcode==445 && buffer[st_i] >0x9f && buffer[st_i] < 0xa8) || 
		(startcode>=0x1c0 && startcode<=0x1cf) ) 
			return true;
	else return false;

}

bool IsAudMpeg (uchar* buffer)
{

	int startcode,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);

	if (startcode>=0x1c0 && startcode<=0x1cf)  
		return true;
	else return false;

}

bool IsSubs (uchar* buffer)
{

	int startcode,st_i,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);
	st_i=0x17+buffer[0x16+stuffing];


	if (startcode==445 && buffer[st_i+stuffing] >0x1f && buffer[st_i+stuffing] < 0x40)
			return true;
	else return false;

}


bool IsSynch (uchar *buffer)
{

	int startcode,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[0]);

	if (startcode==0x1BA) return true;
	else return false;

}

int GetOffSP_DCSQT (uchar* buffer)
{

	int length,st_i,startcode,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);

	st_i=0x17+buffer[0x16+stuffing];

	if (startcode!=445 || buffer[st_i] <=0x1f || buffer[st_i] >= 0x40)
		return 0;
	else
		
	length=GetNbytes(2, &buffer[st_i+3]);

	return length;
}

int GetSubsDataLenght (uchar* buffer)
{

	int length,st_i,startcode,stuffing;

	stuffing=buffer[13]&7;
	startcode=GetNbytes(4, &buffer[14+stuffing]);

	st_i=0x17+buffer[0x16+stuffing];

	if (startcode!=445 || buffer[st_i] <=0x1f || buffer[st_i] >= 0x40)
		return 0;
	else
		
	length=GetNbytes(2, &buffer[st_i+1]);

	return length;
}

int GetPackDataLenght (uchar* buffer)
{

	int length,stuffing;

	stuffing=buffer[13]&7;
	length=GetNbytes(2, &buffer[0x12+stuffing]);

	return length;

}


int getAudId  (uchar *buffer)
{
	int AudId,stuffing;

	stuffing=buffer[13]&7;

	if (!IsAudio(buffer)) return -1;

	if (IsAudMpeg(buffer))
		AudId=buffer[0x11+stuffing];
	else
		AudId=buffer[0x17+buffer[0x16+stuffing]+stuffing];

	return AudId;
}

int getSubId  (uchar *buffer)
{
	int SubId,stuffing;

	stuffing=buffer[13]&7;

	if (!IsSubs(buffer)) return -1;

	SubId=buffer[0x17+buffer[0x16+stuffing]+stuffing];

	return SubId;
}


__int64 MPEGVideoAttr (uchar *buffer)
{
	int stuffing;
	__int64 ret;

	stuffing=buffer[13]&7;

	if (GetNbytes(4,&buffer[0x17+buffer[0x16+stuffing]]) != 0x000001b3 )
		return 0;

	ret=GetNbytes(4,&buffer[0x17+buffer[0x16+stuffing]+4]);

	return ret;

}

int MPEGvert(__int64 i64Attr)
{
   return (int) (i64Attr & 0x000FFF00) >>8;
}

int MPEGhoriz(__int64 i64Attr)
{
   return (int) (i64Attr & 0xFFF00000) >> 20;
}

int MPEGDAR(__int64 i64Attr)
{
   return (int) (i64Attr & 0x0F0) >> 4;

   // 0= forbidden
   // 1= 1:1
   // 2= 4:3
   // 3= 16:9
   // 4= 2,21:1
}

void ModifyLBA (uchar* buffer, __int64 m_i64OutputLBA)
{
	int stuffing;

	stuffing=buffer[13]&7;

// 1st lba number
	buffer[0x30+stuffing]= (uchar)(m_i64OutputLBA%256);
	buffer[0x2F+stuffing]= (uchar)((m_i64OutputLBA/256)%256);
	buffer[0x2E+stuffing]= (uchar)((m_i64OutputLBA/256/256)%256);
	buffer[0x2D+stuffing]= (uchar)(m_i64OutputLBA/256/256/256);

// 2nd lba number
	buffer[0x040E+stuffing]=buffer[0x30+stuffing];
	buffer[0x040D+stuffing]=buffer[0x2F+stuffing];
	buffer[0x040C+stuffing]=buffer[0x2E+stuffing];
	buffer[0x040B+stuffing]=buffer[0x2D+stuffing];
}

void ModifyCID (uchar* buffer, int VobId, int CellId)
{
	int stuffing;

	stuffing=buffer[13]&7;
// VobID
	buffer[0x420+stuffing]=(uchar)(VobId%256);
	buffer[0x41F+stuffing]=(uchar)(VobId/256);
//CellID
	buffer[0x422+stuffing]= (uchar) CellId;
}

bool CleanILV (uchar* buffer)
{
	int stuffing;
	int i;
	bool bWasILV;

	stuffing=buffer[13]&7;

	bWasILV=false;
// Flags
	if ( buffer[0x427+stuffing]!= (uchar)0 || buffer[0x428+stuffing]!=(uchar)0)
	{
		bWasILV=true;
		buffer[0x427+stuffing]=buffer[0x428+stuffing]=(uchar)0;
	}
//End Address
	if ( buffer[0x429+stuffing]!= (uchar)0 || buffer[0x42a+stuffing]!= (uchar)0 ||
		buffer[0x42b+stuffing]!= (uchar)0 || buffer[0x42c+stuffing]!= (uchar)0 )
	{
		bWasILV=true;
		buffer[0x429+stuffing]=buffer[0x42a+stuffing]=buffer[0x42b+stuffing]=buffer[0x42c+stuffing]=(uchar)0;
	}
//Next start
	if ( buffer[0x42d+stuffing]!= (uchar)0 || buffer[0x42e+stuffing]!= (uchar)0 || 
		 buffer[0x42f+stuffing]!= (uchar)0 || buffer[0x430+stuffing]!= (uchar)0 )
	{
		bWasILV=true;
		buffer[0x42d+stuffing]=buffer[0x42e+stuffing]=buffer[0x42f+stuffing]=buffer[0x430+stuffing]=(uchar)0;
	}
// Size
	if ( buffer[0x431+stuffing]!= (uchar)0 || buffer[0x432+stuffing]!= (uchar)0 )
	{
		bWasILV=true;
		buffer[0x431+stuffing]=buffer[0x432+stuffing]=(uchar)0;
	}

// Clear all Angle pointers if WasILV	
	if (bWasILV)
	{
// Non-seamless PCI
		for (i=0x69 ; i< 0x8d ;i++)
			buffer[i+stuffing]=(uchar)0;
// Non-seamless DSI
		for (i=0x4bb ; i< 0x4f1 ;i++)
			buffer[i+stuffing]=(uchar)0;

	}

	return bWasILV;
}


void ModifyPUOPS (uchar* buffer)
{
	int stuffing;

	stuffing=buffer[13]&7;

	buffer[0x35+stuffing]=0;
	buffer[0x36+stuffing]=0;
	buffer[0x37+stuffing]=0;
	buffer[0x38+stuffing]=0;
}



unsigned int ac3bitrate[64]=   {32,32,40,40,48,48,56,56,64,64,
        80,80,96,96,112,112,128,128,160,160,192,192,224,224,256,256,
        320,320,384,384,448,448,512,512,576,576,640,640,
        64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64 };

unsigned int dtsbitrate[32]=   {32,56,64,96,112,128,192,224,256,320,384,448,512,576,640,768,
		960,1024,1152,1280,1344,1408,1411,1472,1536,1920,2048,3072,3840,0,0,0};

unsigned int layerIbitrate[16]=  {0,  32,  64, 96,128,160,192,224,
                                  256,288,320,352,384,416,448, 0};
unsigned int layerIIbitrate[16]= {0,  32,  48, 56, 64, 80, 96,112,
                                  128,160,192,224,256,320,384, 0};
unsigned int layerIIIbitrate[16]={0,  32,  40, 48, 56, 64, 80, 96,
                                  112,128,160,192,224,256,320, 0};


unsigned long int p_bit;
////////////////////////////////////////////////////////////////////////
//      getbits
////////////////////////////////////////////////////////////////////////
unsigned int getbits (int number, unsigned char *p_frame)
{
	unsigned int bit_ini,byte_ini,bit_end,byte_end, output;

	byte_ini=p_bit/8;
	bit_ini=p_bit%8;

	p_bit+=number;

	byte_end=p_bit/8;
	bit_end=p_bit%8;

	if (byte_end==byte_ini)
		output= p_frame[byte_end];
	else if (byte_end==byte_ini+1)
		output= p_frame[byte_end-1]*256 + p_frame[byte_end];
	else
	    output= p_frame[byte_end-2]*256*256+p_frame[byte_end-1]*256+
		      p_frame[byte_end];
		  
	output=(output)>>(8-bit_end);
	output=output & ((1 << number)-1);
    
	return output;
}

int getpcmattr(uchar* buffer)
{
	int start,nbit,ncha,fsample;
	int iAttr;

	start=0x17+buffer[0x16];

	ncha=(buffer[start+5] & 0x7) +1;
	nbit=(buffer[start+5] >> 6) & 0x3;
	fsample=(buffer[start+5] >> 4) & 0x3;

	iAttr=0x8000+(nbit<<6)+(fsample<<4)+(ncha-1);

	return iAttr;
}

int getpcmrate(uchar* buffer)
{
//	int first_acc_p;
	int rate;
	int start,nbit,ncha;

	start=0x17+buffer[0x16];

	ncha=(buffer[start+5] & 0x7) +1;
	nbit=(buffer[start+5] >> 6) & 0x3;

	if (nbit==0) nbit=16;
	else if (nbit==1) nbit=20;
	else if (nbit==2) nbit=24;
	else nbit=0;
//	first_acc_p=GetNbytes(2,&buffer[0x19+buffer[0x16]]);
//	first_acc_p+=(0x1A+buffer[0x16]);


	rate=nbit*48*ncha;
	return rate;

}

int getac3attr(uchar* buffer)
{
	int first_acc_p;
	int frmsizecod,fscod,acmod, lfeon,ncha;
	int iAttr;

	first_acc_p=GetNbytes(2,&buffer[0x19+buffer[0x16]]);
	first_acc_p+=(0x1A+buffer[0x16]);

	if (buffer[first_acc_p+0]!= 0x0B || buffer[first_acc_p+1]!=0x77)
		return -2;

	frmsizecod=buffer[first_acc_p+4] & 0x3F;
	fscod=(buffer[first_acc_p+4]>>6) & 0x3;
	if (fscod !=0) 
		return -2;
	acmod=(buffer[first_acc_p+6]>>5) & 0x7;
	if (acmod < 2) 
		lfeon=(buffer[first_acc_p+6]>>4) & 0x1;
	else if (acmod < 5 || acmod == 6 )
		lfeon=(buffer[first_acc_p+6]>>2) & 0x1;
	else if (acmod == 5 || acmod == 7 )
		lfeon=buffer[first_acc_p+6] & 0x1;

	ncha=acmod;
	if (acmod ==0) ncha=2;
	else if (acmod >=6) ncha=acmod-2;
	else if (acmod >=4) ncha=acmod-1;
	if (lfeon) ncha++;

/*
	if      (acmod==0) strcpy (mode,"1+1: A+B");
	else if (acmod==1) strcpy (mode,"1/0: C");
	else if (acmod==2) strcpy (mode,"2/0: L+R");
	else if (acmod==3) strcpy (mode,"3/0: L+C+R");
	else if (acmod==4) strcpy (mode,"2/1: L+R+S");
	else if (acmod==5) strcpy (mode,"3/1: L+C+R+S");
	else if (acmod==6) strcpy (mode,"2/2: L+R+SL+SR");
	else if (acmod==7) strcpy (mode,"3/2: L+C+R+SL+SR");
*/

	iAttr=0x0000 + (ncha-1);

	return iAttr;
}

int getac3rate(uchar* buffer)
{
	int first_acc_p,rate;

//	p_bit=0;

	first_acc_p=GetNbytes(2,&buffer[0x19+buffer[0x16]]);
	first_acc_p+=(0x1A+buffer[0x16]);

	if (buffer[first_acc_p+0]!= 0x0B || buffer[first_acc_p+1]!=0x77)
		return 0;

	rate=ac3bitrate[buffer[first_acc_p+4] & 0x3F];
	return rate;

}

int getdtsattr( uchar* buffer)
{

	int first_acc_p;
	int amode, ncha, sfreq,lfeon;
	int unused;
	int iAttr;

//		fsample=((double)(dtsfsample[sfreq]))/1000.0;

	p_bit=0;

	first_acc_p=GetNbytes(2,&buffer[0x19+buffer[0x16]]);
	first_acc_p+=(0x1A+buffer[0x16]);

	if (buffer[first_acc_p+0]!= 0x7F || buffer[first_acc_p+1]!=0xFE ||
		buffer[first_acc_p+2]!= 0x80 || buffer[first_acc_p+3]!=0x01 )
		return -2;

	unused=   getbits (32,&buffer[first_acc_p]);
	unused=   getbits (1, &buffer[first_acc_p]);
	unused=   getbits (5, &buffer[first_acc_p]);
	unused=   getbits (1, &buffer[first_acc_p]);
	unused=   getbits (7, &buffer[first_acc_p]);
	unused=   getbits (14,&buffer[first_acc_p]);
	amode=    getbits (6, &buffer[first_acc_p]); 
	sfreq=    getbits (4, &buffer[first_acc_p]);
	unused=   getbits (5, &buffer[first_acc_p]);
	unused=	  getbits (10,&buffer[first_acc_p]);
	lfeon=	  getbits (1, &buffer[first_acc_p]);

//	if (sfreq !=13) return -2;

/*	if		(amode==0)		fileinfo->csMode="Mono";
	else if (amode==1)		fileinfo->csMode="A+B (Dual Mono)";
	else if (amode==2)		fileinfo->csMode="L+R (Stereo)";
	else if (amode==3)		fileinfo->csMode="(L+R) + (L-R): (Sum + Diff)";
	else if (amode==4)		fileinfo->csMode="LT + RT ";
	else if (amode==5)		fileinfo->csMode="C+L+R";
	else if (amode==6)		fileinfo->csMode="L+R+S";
	else if (amode==7)		fileinfo->csMode="C+L+R+S";
	else if (amode==8)		fileinfo->csMode="L+R+SL+SR";
	else if (amode==9)		fileinfo->csMode="C+L+R+SL+SR";
	else if (amode==10)		fileinfo->csMode="CL+CR+L+R+SL+SR";
	else if (amode==11)		fileinfo->csMode="C+L+R+LR+RR+OV";
	else if (amode==12)		fileinfo->csMode="CF+CR+LF+RF+LR+RR";
	else if (amode==13)		fileinfo->csMode="CL+C+CR+L+R+SL+SR";
	else if (amode==14)		fileinfo->csMode="CL+CR+L+R+SL1+SL2+SR1+SR2";
	else if (amode==15)		fileinfo->csMode="CL+C+CR+L+R+SL+S+SR";
	else 					fileinfo->csMode="User defined";
*/
	if (amode ==0) ncha=1;
	else if (amode <=4 ) ncha =2;
	else if (amode <=6 ) ncha =3;
	else if (amode <=8 ) ncha =4;
	else if (amode <=9 ) ncha =5;
	else if (amode <=12 ) ncha =6;
	else if (amode <=13 ) ncha =7;
	else if (amode <=15 ) ncha =8;

	if (lfeon) ncha++;

	iAttr= 0xC000+ (ncha-1);
	return iAttr;
}

int getdtsrate( uchar* buffer)
{

	int first_acc_p,rate;
	int unused;

	p_bit=0;

	first_acc_p=GetNbytes(2,&buffer[0x19+buffer[0x16]]);
	first_acc_p+=(0x1A+buffer[0x16]);

	if (buffer[first_acc_p+0]!= 0x7F || buffer[first_acc_p+1]!=0xFE ||
		buffer[first_acc_p+2]!= 0x80 || buffer[first_acc_p+3]!=0x01 )
		return 0;

	unused=   getbits (32, &buffer[first_acc_p]); 
	unused=   getbits (1, &buffer[first_acc_p]);
	unused=   getbits (5, &buffer[first_acc_p]);
	unused=   getbits (1, &buffer[first_acc_p]);
	unused=   getbits (7, &buffer[first_acc_p]);
	unused=   getbits (14,&buffer[first_acc_p]);
	unused=   getbits (6, &buffer[first_acc_p]); 
	unused=   getbits (4, &buffer[first_acc_p]);

	rate=     getbits (5, &buffer[first_acc_p]);
	rate=dtsbitrate[rate];

	return rate;
}

int getmpglayer( uchar* buffer)
{

	int first_acc_p,layer;

	first_acc_p=0x17+buffer[0x16];

	if (buffer[first_acc_p+0]!= 0xFF || (buffer[first_acc_p+1] & 0xF0) !=0xF0)
		return 0;

	layer=(buffer[first_acc_p+1] & 0x6)>>1;
	if (layer==3) layer=1;
	else if (layer==1) layer=3;

	return layer;
}

int getmpgattr( uchar* buffer)
{

	int first_acc_p;
	int mode,fsamp,layer;
	int ncha,iAttr;

	first_acc_p=0x17+buffer[0x16];

	if (buffer[first_acc_p+0]!= 0xFF || (buffer[first_acc_p+1] & 0xF0) !=0xF0)
		return -2;

	layer=(buffer[first_acc_p+1] & 0x6)>>1;
	if (layer==3) layer=1;
	else if (layer==1) layer=3;

	fsamp=(buffer[first_acc_p+2] >> 2) & 0x3;
	mode =(buffer[first_acc_p+3] >> 6) & 0x3;

/*
	For Layer I the following equation is valid: 
	N = 12 * bit_rate / sampling_frequency.
	For Layers II and III the equation becomes: 
	N = 144 * bit_rate / sampling_frequency.
	 384000 / 44100

	unused=		getbits (12, caracter);
	unused=		getbits (1, caracter);
	layer=		getbits (2, caracter);
	unused=		getbits (1, caracter);
	unused=		getbits (4, caracter);
	fsamp=		getbits (2,caracter);
	unused=		getbits (1, caracter);
	unused=		getbits (1, caracter);
	mode=		getbits (2, caracter);
	if		(mode==0)		fileinfo->csMode="Stereo";
	else if (mode==1)		fileinfo->csMode="Joint Stereo";
	else if (mode==2)		fileinfo->csMode="Dual Channel (Dual Mono)";
	else if (mode==3)		fileinfo->csMode="Single Channel (Mono)";

*/
	ncha=2;
	if (mode==3) ncha=1;

	iAttr=0x4000+ (ncha-1);

	return iAttr;

}
int getmpgrate( uchar* buffer)
{

	int first_acc_p,rate;
	int rateidx,layer;

	first_acc_p=0x17+buffer[0x16];

	if (buffer[first_acc_p+0]!= 0xFF || (buffer[first_acc_p+1] & 0xF0) !=0xF0)
		return 0;

	layer=(buffer[first_acc_p+1] & 0x6)>>1;
	if (layer==3) layer=1;
	else if (layer==1) layer=3;
/*
	For Layer I the following equation is valid: 
	N = 12 * bit_rate / sampling_frequency.
	For Layers II and III the equation becomes: 
	N = 144 * bit_rate / sampling_frequency.
	 384000 / 44100
*/
	rateidx=  (buffer[first_acc_p+2]>>4) & 0xF;

	rate=0;
	if (layer==3)
		rate=layerIIIbitrate[rateidx];
	else if (layer==2)
		rate=layerIIbitrate[rateidx];
	else if (layer==1)
		rate=layerIbitrate[rateidx];

	return rate;

}

int getaudattr( uchar* buffer)
{
	int iRet,iAudID;

	iAudID=getAudId(buffer);

	if (iAudID==-1) return -2;

	if      (iAudID >=0x80 && iAudID <=0x87)  iRet=getac3attr(buffer);
	else if (iAudID >=0x88 && iAudID <=0x8f)  iRet=getdtsattr(buffer);
	else if (iAudID >=0xa0 && iAudID <=0xa7)  iRet=getpcmattr(buffer);
	else if (iAudID >=0xc0 && iAudID <=0xc7)  iRet=getmpgattr(buffer);

	return iRet;
}

char *getaudiocode(int iAttr)
{
	static char code[30];

	strcpy(code,"unknown"); 

	if      ((iAttr & 0xE000) == 0x0000) strcpy(code,"AC3"); 
	else if ((iAttr & 0xE000) == 0x2000) strcpy(code,"unknown"); 
	else if ((iAttr & 0xE000) == 0x4000) strcpy(code,"MPEG"); 
	else if ((iAttr & 0xE000) == 0x6000) strcpy(code,"MPEG"); 
	else if ((iAttr & 0xE000) == 0x8000) strcpy(code,"LPCM"); 
	else if ((iAttr & 0xE000) == 0xa000) strcpy(code,"unknown"); 
	else if ((iAttr & 0xE000) == 0xc000) strcpy(code,"DTS"); 
	else if ((iAttr & 0xE000) == 0xe000) strcpy(code,"unknown"); 

	return &code[0];
}

int getpcmaudiobits(int iAttr)
{
	int nbit;

	nbit=(iAttr>>6)&0x3;

	if (nbit==0) nbit=16;
	else if (nbit==1) nbit=20;
	else if (nbit==2) nbit=24;
	else nbit=0;

	return nbit;
}

int getpcmaudiofsample(int iAttr)
{
	int fsample;

	fsample=(iAttr>>4) & 0x3;

	if  (fsample==0) fsample=48000;
	else fsample=96000;

	return fsample;
}

unsigned char uVTSM_PGCI_UT[48] = {
	0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x0B, 0x65, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 
	0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFB, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

unsigned char EndSeqPlusPadding[64] = {
	0x00, 0x00, 0x01, 0xBA, 0x44, 0x00, 0x04, 0x12, 0x4C, 0x55, 0x01, 0x89, 0xC3, 0xF8, 0x00, 0x00, 
	0x01, 0xE0, 0x00, 0x07, 0x81, 0x00, 0x00, 0x00, 0x00, 0x01, 0xB7, 0x00, 0x00, 0x01, 0xBE, 0x07, 
	0xDF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};


unsigned char PaddingPack[32] = {
	0x00, 0x00, 0x01, 0xBA, 0x44, 0x00, 0x04, 0x00, 0x04, 0x01, 0x01, 0x89, 0xC3, 0xF8, 0x00, 0x00,
	0x01, 0xBE, 0x07, 0xEC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};

void FillPaddingPack(uchar * buffer)
{
	int k;

	for (k=0; k<32;k++)
		buffer[k]= PaddingPack[k];
	for (k=32; k<2048;k++)
		buffer[k]= 0xFF;

}
	
void FillSimpleVob(uchar *SimpleVob, int iVideoAttr)
{
// Prepare SimpleVob
// DAR is hacked in MPEG header, but resolution cannot be hacked,
//	it will not work, so different simple Vobs have been created.
// VCD resolution is coded in MPEG-2. All the others are in MPEG-2

	int k;
	__int64 i64AttrMPEG;
	int iFPS;

	if (((iVideoAttr & 0x1000)>>12) == PAL )
	{
		if ((iVideoAttr & 0x18) == 0 )   
			for (k=0; k<2048*5;k++)	SimpleVob[k]= PalSimpleVob720x576[k];
		if ((iVideoAttr & 0x18) == 0x08 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= PalSimpleVob704x576[k];
		if ((iVideoAttr & 0x18) == 0x10 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= PalSimpleVob352x576[k];
		if ((iVideoAttr & 0x18) == 0x18 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= PalSimpleVob352x288[k];
	}
	else
	{
		if ((iVideoAttr & 0x18) == 0 )   
			for (k=0; k<2048*5;k++)	SimpleVob[k]= NtscSimpleVob720x480[k];
		if ((iVideoAttr & 0x18) == 0x08 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= NtscSimpleVob704x480[k];
		if ((iVideoAttr & 0x18) == 0x10 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= NtscSimpleVob352x480[k];
		if ((iVideoAttr & 0x18) == 0x18 ) 
			for (k=0; k<2048*5;k++)	SimpleVob[k]= NtscSimpleVob352x240[k];
	}


	i64AttrMPEG=GetNbytes(4,&SimpleVob[0x17+SimpleVob[0x16+2048]+4+2048]);
	iFPS= (int) (i64AttrMPEG & 0xF);

	i64AttrMPEG= i64AttrMPEG & 0xFFFFFF00;

	if ((iVideoAttr & 0x0C00) == 0x0C00 )
		//16:9
		i64AttrMPEG+=0x30;
	else if ((iVideoAttr & 0x0C00) == 0 )
		//4:3
		i64AttrMPEG+=0x20;

	i64AttrMPEG+=iFPS;

	Put4bytes (i64AttrMPEG, &SimpleVob[0x17+SimpleVob[0x16+2048]+4+2048]);

}


//*************************************************************
//
//  RegDelnodeRecurse()
//
//  Purpose:    Deletes a registry key and all it's subkeys / values.
//
//  Parameters: hKeyRoot    -   Root key
//              lpSubKey    -   SubKey to delete
//
//  Return:     TRUE if successful.
//              FALSE if an error occurs.
//
//*************************************************************

BOOL RegDelnodeRecurse (HKEY hKeyRoot, LPTSTR lpSubKey)
{
    LPTSTR lpEnd;
    LONG lResult;
    DWORD dwSize;
    TCHAR szName[MAX_PATH];
    HKEY hKey;
    FILETIME ftWrite;

    // First, see if we can delete the key without having
    // to recurse.

    lResult = RegDeleteKey(hKeyRoot, lpSubKey);

    if (lResult == ERROR_SUCCESS) 
        return TRUE;

    lResult = RegOpenKeyEx (hKeyRoot, lpSubKey, 0, KEY_READ, &hKey);

    if (lResult != ERROR_SUCCESS) 
    {
        if (lResult == ERROR_FILE_NOT_FOUND) {
            printf("Key not found.\n");
            return TRUE;
        } 
        else {
            printf("Error opening key.\n");
            return FALSE;
        }
    }

    // Check for an ending slash and add one if it is missing.

    lpEnd = lpSubKey + lstrlen(lpSubKey);

    if (*(lpEnd - 1) != TEXT('\\')) 
    {
        *lpEnd =  TEXT('\\');
        lpEnd++;
        *lpEnd =  TEXT('\0');
    }

    // Enumerate the keys

    dwSize = MAX_PATH;
    lResult = RegEnumKeyEx(hKey, 0, szName, &dwSize, NULL,
                           NULL, NULL, &ftWrite);

    if (lResult == ERROR_SUCCESS) 
    {
        do {

            lstrcpy (lpEnd, szName);

            if (!RegDelnodeRecurse(hKeyRoot, lpSubKey)) {
                break;
            }

            dwSize = MAX_PATH;

            lResult = RegEnumKeyEx(hKey, 0, szName, &dwSize, NULL,
                                   NULL, NULL, &ftWrite);

        } while (lResult == ERROR_SUCCESS);
    }

    lpEnd--;
    *lpEnd = TEXT('\0');

    RegCloseKey (hKey);

    // Try again to delete the key.

    lResult = RegDeleteKey(hKeyRoot, lpSubKey);

    if (lResult == ERROR_SUCCESS) 
        return TRUE;

    return FALSE;
}

//*************************************************************
//
//  RegDelnode()
//
//  Purpose:    Deletes a registry key and all it's subkeys / values.
//
//  Parameters: hKeyRoot    -   Root key
//              lpSubKey    -   SubKey to delete
//
//  Return:     TRUE if successful.
//              FALSE if an error occurs.
//
//*************************************************************

BOOL RegDelnode (HKEY hKeyRoot, LPTSTR lpSubKey)
{
    TCHAR szDelKey[2 * MAX_PATH];

    lstrcpy (szDelKey, lpSubKey);
    return RegDelnodeRecurse(hKeyRoot, szDelKey);

}
